/* ===========================================================================

    lcd.c  - LCD routines for Axiom CMLU-12C32

    Version:        1.0
    Author:         Axiom Manufacturing (www.axman.com)
    Compiler:       CodeWarrior S12(X) v5.1

    This is freeware - use as you like

==============================================================================
------------------------------------------------------------------------------
*/

#include "derivative.h"			// derivative-specific definitions 
#include "lcd.h"
#include "sci_hc12.h"			// hex2char function


// RAM Variables
// -------------
unsigned char LCDBuf;   // holds data and status bits sent to LCD
unsigned char LCDStat;  // holds LCD status

#define LCD_DELAYTIME   0x200   // adjust this value Lower for quicker LCD performance, Higher if you're seeing garbage on the display

// simple delay loop, waits the specified number of counts
void LCD_delayu(unsigned int ucount){
    while(ucount > 0){
        --ucount;
        // look at your compiler output and count the number of cycles used.
        // add more of these if needed to fine tune the exact delay you want
        //        asm("nop");

    }
}
// simple delay loop, LCD_delayu() * LCD_DELAYTIME
void LCD_delaym(unsigned int mcount){
    while(mcount > 0){
        --mcount;
        LCD_delayu(LCD_DELAYTIME);
    }
}

// Simple Serial Driver (SPI) send byte
// sends data byte in global LCDBuf
// return received back value in global LCDStat
void LCDSend(){
    LCD_delaym(1);
    LCDStat = SPISR;				// clear status of SPI by reading it
    SPIDR = LCDBuf;				// send byte

    do{ // wait for status flag to go high
        LCDStat = SPISR;
    }while(LCDStat < 0x80);

    LCDStat = SPIDR;				// receive value back
}

// writes 4 bit data to lcd port
void lcd_wr_4(char LCDdata){
    // merge lower 4 bits of LCDdata with upper 4 bits of LCDBuf (control bits)
    LCDBuf &= 0xF0;
    LCDBuf |= LCDdata;

    LCDBuf &= ~LCD_EN;			// enable low
    LCDSend();						// send the data
    LCDBuf |= LCD_EN;			// enable high
    LCDSend();						// send the data
    LCDBuf &= ~LCD_EN;			// enable low
    LCDSend();						// send the data
}

// same as above but with delay
void lcd_wr_4d(char LCDdata){
    lcd_wr_4(LCDdata);
    LCD_delaym(50);
}
// Lcd Write 8 bit Data , upper 4 bits first, then lower
void LCD_Write(unsigned char lcdval){
    lcd_wr_4(lcdval >> 4);		// send upper 4 bits
    lcd_wr_4(lcdval & 0x0F);	// send lower 4 bits
}
// write a COMMAND byte to the LCD
void lcd_cmd(unsigned char cmdval){
    LCDBuf &= ~LCD_RS;			// clear RS to select LCD Command mode
    LCD_Write(cmdval);
    LCD_delaym(10);
}

/*----------------------------------------------------------------------------
LCDputch
--------
 Send a character to the LCD for display.

 INPUT:     datval      character to display
----------------------------------------------------------------------------*/
void LCDputch(unsigned char datval){
    LCDBuf |= LCD_RS; // set RS to select LCD Data mode
    LCD_Write(datval); // write a DATA byte to the LCD
}

/*----------------------------------------------------------------------------
LCDputs
-------
 Send a string of characters to the LCD.  The string must end in a 0x00

 INPUT:		sptr		pointer to the string
----------------------------------------------------------------------------*/
void LCDputs(char *sptr){
	while(*sptr){
        LCDputch(*sptr);
		++sptr;
	}
}

/*----------------------------------------------------------------------------
LCDInit
-------
 Initialize LCD port

 ----------------------------------------------------------------------------*/
void LCDInit(){

    //  Turn on Spi
    SPICR1 = 0x52;
    SPICR2 = 0x10;				// enable /SS
    SPIBR = 0x03;					// Spi baud clock = busclock / 16
    LCDBuf = (LCD_WR + LCD_EN);	// set WR and EN bits
    LCDSend();						// send status to LCD

    // Initialize LCD
    LCDBuf &= ~(LCD_RS + LCD_EN);	// clear RS and EN bits (select lcd Command)
    LCDSend();						// send status to LCD

    // delay's are used because this lcd interface does not provide status
    LCD_delaym(50);

    // set to 4 bit wide mode
    lcd_wr_4d(3);					// send 3
    lcd_wr_4d(3);					// send 3
    lcd_wr_4d(3);					// send 3
    lcd_wr_4d(2);					// send 2

    lcd_cmd(0x2c);				// 2x40 display
    lcd_cmd(0x06);				// display and cursor on
    lcd_cmd(0x0e);				// shift cursor right
    lcd_cmd(0x01);				// clear display and home cursor
    lcd_cmd(0x80);

    LCDBuf = 0;					// Reset Lcd states to rest
    LCDSend();						// send status to LCD
}


/*******************************************************************************
outhexLCD

	formats 8-bit hex value into ASCII for display on LCD.  Use temp var to track
	cursor position.
	
	input: 8-bit hex value 
	outpt: ASCII representation of input to LCD
********************************************************************************/
void LCDouthex(byte hval){
	// convert HEX value input to ASCII text output

	unsigned char unib,lnib;			// temp variable for upper and lower nibbles
	
	unib = (hval >> 4);					// high nibble
	lnib = hval & 0x0F;					// low nibble

	LCDputch(hex2char(unib));				// send char out selected COM port
	LCDputch(hex2char(lnib));				// send char out selected COM port	
}  


/******************************************************************************
LCD_Cursor_Right(byte steps)

	move LCD cursor to the right

******************************************************************************/
void	LCD_Cursor_Right(byte steps){
	
	byte n;
	
	for(n=0;n<steps;n++){
		lcd_cmd(0x14);
	}
}

/******************************************************************************
LCD_Cursor_Left(byte steps)

	move LCD cursor to the left

******************************************************************************/
void	LCD_Cursor_Left(byte steps){
	
	byte n;
	
	for(n=0;n<steps;n++){
		lcd_cmd(0x10);
	}
}


/******************************************************************************
LCD_Home(void)

	move LCD cursor to home position

******************************************************************************/
void LCD_Home(void){
	lcd_cmd(0x02);				// go home
}